@@ -1,13 +1,13 @@
 use strict;
 use warnings;
 package BenchmarkAnything::Storage::Frontend::Lib;
-# git description: v0.004-30-gb054d29
+# git description: v0.005-6-ga3e7734
 
 BEGIN {
   $BenchmarkAnything::Storage::Frontend::Lib::AUTHORITY = 'cpan:SCHWIGON';
 }
 # ABSTRACT: Basic functions to access a BenchmarkAnything store
-$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.005';
+$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.006';
 use Scalar::Util 'reftype';
 
 
@@ -15,8 +15,8 @@ sub new
 {
         my $class = shift;
         my $self  = bless { @_ }, $class;
-        $self->_read_config;
-        $self->connect;
+        $self->_read_config unless $self->{noconfig};
+        $self->connect      unless $self->{noconnect};
         return $self;
 }
 
@@ -186,14 +186,20 @@ sub _read_config
         require YAML::Any;
 
         # don't look into user's homedir if we are running tests
-        my $default_cfgfile = $ENV{HARNESS_ACTIVE} ? undef : File::HomeDir->my_home . "/.benchmarkanything.cfg";
+        my $default_cfgfile = $ENV{HARNESS_ACTIVE} ? undef : File::HomeDir->my_home . "/.benchmarkanything/default.cfg";
 
         # read file
-        $self->{cfgfile} = $self->{cfgfile} || $ENV{BENCHMARKANYTHING_CONFIGFILE} || $default_cfgfile;
-        $self->{config}  = YAML::Any::Load("".File::Slurp::read_file($self->{cfgfile}));
+        eval {
+                $self->{cfgfile} = $self->{cfgfile} || $ENV{BENCHMARKANYTHING_CONFIGFILE} || $default_cfgfile;
+                $self->{config}  = YAML::Any::Load("".File::Slurp::read_file($self->{cfgfile}));
+        };
+        if ($@)
+        {
+                die "benchmarkanything: error loading configfile: $@\n";
+        }
 
         # defaults
-        $self->{config}{benchmarkanything}{frontend} ||= 'lib';
+        $self->{config}{benchmarkanything}{backend} ||= 'local';
 
         return $self;
 }
@@ -203,8 +209,8 @@ sub connect
 {
         my ($self) = @_;
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 require DBI;
                 require Tapper::Benchmark;
@@ -212,9 +218,9 @@ sub connect
 
                 # connect
                 print "Connect db...\n" if $self->{verbose};
-                my $dsn      = $self->{config}{benchmarkanything}{backends}{tapper}{benchmark}{dsn};
-                my $user     = $self->{config}{benchmarkanything}{backends}{tapper}{benchmark}{user};
-                my $password = $self->{config}{benchmarkanything}{backends}{tapper}{benchmark}{password};
+                my $dsn      = $self->{config}{benchmarkanything}{storage}{tapper}{benchmark}{dsn};
+                my $user     = $self->{config}{benchmarkanything}{storage}{tapper}{benchmark}{user};
+                my $password = $self->{config}{benchmarkanything}{storage}{tapper}{benchmark}{password};
                 my $dbh      = DBI->connect($dsn, $user, $password, {'RaiseError' => 1})
                  or die "benchmarkanything: can not connect: ".$DBI::errstr;
 
@@ -222,7 +228,7 @@ sub connect
                 $self->{dbh}              = $dbh;
                 $self->{tapper_benchmark} = Tapper::Benchmark->new({dbh => $dbh, debug => $self->{debug} });
         }
-        elsif ($frontend eq 'http')
+        elsif ($backend eq 'http')
         {
                 my $ua  = $self->_get_user_agent;
                 my $url = $self->_get_base_url."/api/v1/hello";
@@ -237,8 +243,8 @@ sub disconnect
 {
         my ($self) = @_;
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 if ($self->{dbh}) {
                         $self->{dbh}->commit unless $self->{dbh}{AutoCommit};
@@ -254,7 +260,7 @@ sub _are_you_sure
         my ($self) = @_;
 
         # DSN
-        my $dsn = $self->{config}{benchmarkanything}{backends}{tapper}{benchmark}{dsn};
+        my $dsn = $self->{config}{benchmarkanything}{storage}{tapper}{benchmark}{dsn};
 
         # option --really
         if ($self->{really})
@@ -295,7 +301,7 @@ sub createdb
                 my $batch            = DBIx::MultiStatementDo->new(dbh => $self->{dbh});
 
                 # get schema SQL according to driver
-                my $dsn      = $self->{config}{benchmarkanything}{backends}{tapper}{benchmark}{dsn};
+                my $dsn      = $self->{config}{benchmarkanything}{storage}{tapper}{benchmark}{dsn};
                 my ($scheme, $driver, $attr_string, $attr_hash, $driver_dsn) = DBI->parse_dsn($dsn)
                  or die "benchmarkanything: can not parse DBI DSN '$dsn'";
                 my ($dbname) = $driver_dsn =~ m/database=(\w+)/g;
@@ -316,6 +322,103 @@ sub createdb
 }
 
 
+sub init_workdir
+{
+        my ($self) = @_;
+
+        require File::Basename;
+        require File::ShareDir;
+        require File::HomeDir;
+        require File::Slurp;
+
+        my $home_ba = File::HomeDir->my_home."/.benchmarkanything";
+        my $command = File::Basename::basename($0);
+
+        if (-d $home_ba)
+        {
+                print "Workdir '$home_ba' already exists - skipping.\n" if $self->{verbose};
+        }
+        else
+        {
+                require File::Path;
+                File::Path::make_path($home_ba);
+        }
+
+        foreach my $basename (qw(client.cfg server.cfg default.cfg README))
+        {
+                my $source_file = File::ShareDir::dist_file('BenchmarkAnything-Storage-Frontend-Lib', "config/$basename");
+                my $dest_file   = "$home_ba/$basename";
+
+                if (! -e $dest_file)
+                {
+                        my $content     =  File::Slurp::read_file($source_file);
+
+                        # poor man's templating
+                        $content        =~ s{\[%\s*CLIENTCFG\s*%\]}{$home_ba/client.cfg}g;
+                        $content        =~ s{\[%\s*SERVERCFG\s*%\]}{$home_ba/server.cfg}g;
+                        $content        =~ s{\[%\s*LOCALCFG\s*%\]}{$home_ba/default.cfg}g;
+                        $content        =~ s{\[%\s*CFG\s*%\]}{$dest_file}g;
+                        $content        =~ s{\[%\s*HOME\s*%\]}{$home_ba}g;
+
+                        print "Create configfile: $dest_file...\n" if $self->{verbose};
+                        open my $CFGFILE, ">", $dest_file or die "Could not create $dest_file.\n";
+                        print $CFGFILE $content;
+                        close $CFGFILE;
+                }
+                else
+                {
+                        print "Config '$dest_file' already exists - skipping.\n" if $self->{verbose};
+                }
+        }
+
+        my $dbfile = "$home_ba/benchmarkanything.sqlite";
+        my $we_created_db = 0;
+        if (! -e $dbfile)
+        {
+                print "Create storage: $dbfile...\n" if $self->{verbose};
+                __PACKAGE__->new(cfgfile => "$home_ba/default.cfg",
+                                 really  => "dbi:SQLite:$dbfile",
+                                )->createdb;
+                $we_created_db = 1;
+        }
+        else
+        {
+                print "Storage '$dbfile' already exists - skipping.\n" if $self->{verbose};
+        }
+
+        if ($self->{verbose})
+        {
+                print "\n";
+                print "By default it will use this config: $home_ba/default.cfg\n";
+                print "If you want another one, set it in your ~/.bash_profile:\n";
+                print "  export BENCHMARKANYTHING_CONFIGFILE=$home_ba/client.cfg\n";
+
+                unless ($we_created_db)
+                {
+                        print "\n";
+                        print "Initialize a new database (it asks for confirmation) with:\n";
+                        print "  $command createdb\n";
+                        print "\nReady.\n";
+                }
+                else
+                {
+                        print "\n";
+                        print "Create sample values like this:\n";
+                        print qq(  echo '{"BenchmarkAnythingData":[{"NAME":"benchmarkanything.hello.world", "VALUE":17.2}]}' | $command add\n);
+                        print "\n";
+                        print "List metric names:\n";
+                        print qq(  $command listnames\n);
+                        print "\n";
+                        print "Query sample values:\n";
+                        print qq(  echo '{"select":["NAME","VALUE"],"where":[["=","NAME","benchmarkanything.hello.world"]]}' | $command search\n);
+                        print "\n";
+                }
+        }
+
+        return;
+}
+
+
 sub add
 {
         my ($self, $data) = @_;
@@ -337,19 +440,19 @@ sub add
 
         # --- add to storage ---
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 my $success;
                 if ($self->{queuemode})
                 {
                         # only queue for later processing
-                        print "Enqueue data [frontend:lib]...\n" if $self->{verbose} or $self->{debug};
+                        print "Enqueue data [backend:local]...\n" if $self->{verbose} or $self->{debug};
                         $success = $self->{tapper_benchmark}->enqueue_multi_benchmark($data->{BenchmarkAnythingData});
                 }
                 else
                 {
-                        print "Add data [frontend:lib]...\n" if $self->{verbose} or $self->{debug};
+                        print "Add data [backend:local]...\n" if $self->{verbose} or $self->{debug};
                         # preserve order, otherwise add_multi_benchmark() would reorder to optimize insert
                         foreach my $chunk (@{$data->{BenchmarkAnythingData}})
                         {
@@ -363,11 +466,11 @@ sub add
                 }
                 print "Done.\n" if $self->{verbose} or $self->{debug};
         }
-        elsif ($frontend eq 'http')
+        elsif ($backend eq 'http')
         {
                 my $ua  = $self->_get_user_agent;
                 my $url = $self->_get_base_url."/api/v1/add";
-                print "Add data [frontend:http]...\n" if $self->{verbose} or $self->{debug};
+                print "Add data [backend:http]...\n" if $self->{verbose} or $self->{debug};
                 my $res = $ua->post($url => json => $data)->res;
                 print "Done.\n" if $self->{verbose} or $self->{debug};
 
@@ -375,7 +478,7 @@ sub add
         }
         else
         {
-                die "benchmarkanything: no frontend '$frontend', available frontends are: 'http', 'lib'.\n";
+                die "benchmarkanything: no backend '$backend', available backends are: 'http', 'local'.\n";
         }
 
         return $self;
@@ -394,7 +497,7 @@ sub _get_user_agent
 
 sub _get_base_url
 {
-        shift->{config}{benchmarkanything}{frontends}{http}{base_url};
+        shift->{config}{benchmarkanything}{backends}{http}{base_url};
 }
 
 
@@ -408,14 +511,14 @@ sub search
                 die "benchmarkanything: no query or value_id provided.\n";
         }
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 # single values
                 return $self->{tapper_benchmark}->get_single_benchmark_point($value_id) if $value_id;
                 return $self->{tapper_benchmark}->search_array($query);
         }
-        elsif ($frontend eq 'http')
+        elsif ($backend eq 'http')
         {
                 my $ua  = $self->_get_user_agent;
                 my $url = $self->_get_base_url."/api/v1/search";
@@ -433,7 +536,7 @@ sub search
         }
         else
         {
-                die "benchmarkanything: no frontend '$frontend', available frontends are: 'http', 'lib'.\n";
+                die "benchmarkanything: no backend '$backend', available backends are: 'http', 'local'.\n";
         }
 }
 
@@ -442,12 +545,12 @@ sub listnames
 {
         my ($self, $pattern) = @_;
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 return $self->{tapper_benchmark}->list_benchmark_names(defined($pattern) ? ($pattern) : ());
         }
-        elsif ($frontend eq 'http')
+        elsif ($backend eq 'http')
         {
                 my $ua  = $self->_get_user_agent;
                 my $url = $self->_get_base_url."/api/v1/listnames";
@@ -462,7 +565,7 @@ sub listnames
         }
         else
         {
-                die "benchmarkanything: no frontend '$frontend', available frontends are: 'http', 'lib'.\n";
+                die "benchmarkanything: no backend '$backend', available backends are: 'http', 'local'.\n";
         }
 }
 
@@ -471,8 +574,8 @@ sub gc
 {
         my ($self) = @_;
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 $self->{tapper_benchmark}->gc;
         }
@@ -485,8 +588,8 @@ sub process_raw_result_queue
 
         $count ||= 10;
 
-        my $frontend = $self->{config}{benchmarkanything}{frontend};
-        if ($frontend eq 'lib')
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
         {
                 my $dequeued_raw_bench_bundle_id;
                 do {
@@ -496,7 +599,7 @@ sub process_raw_result_queue
         }
         else
         {
-                die "benchmarkanything: only frontend 'lib' allowed here.\n";
+                die "benchmarkanything: only backend 'local' allowed here.\n";
         }
         return;
 }
@@ -860,6 +963,11 @@ operation it verifies an "are you sure" test, by comparing the
 parameter C<really> against the DSN from the config, or if that
 doesn't match, asking interactively on STDIN.
 
+=head2 init_workdir
+
+Initializes a work directory C<~/.benchmarkanything/> with config
+files, which should work by default and can be tweaked by the user.
+
 =head2 add ($data)
 
 Adds all data points of a BenchmarkAnything structure to the backend
